<?php

namespace Drupal\eca_content\Plugin\Action;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\eca\Plugin\Action\ActionBase;
use Drupal\eca\PluginConditionBase;

/**
 * Replaces Drupal\Core\Field\FieldUpdateActionBase.
 *
 * We need to replace the core base class because within the ECA context
 * entities should not be saved after modifying a field value.
 *
 * The replacement is achieved with PHP's class_alias(), see eca_content.module
 */
abstract class FieldUpdateActionBase extends ActionBase {

  /**
   * Gets an array of values to be set.
   *
   * @return array
   *   Array of values with field names as keys.
   */
  abstract protected function getFieldsToUpdate();

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [
      'load_entity_into_token' => TRUE,
      'load_current_user_into_token' => TRUE,
      'save_entity' => FALSE,
      'strip_tags' => FALSE,
      'trim' => FALSE,
    ] + parent::defaultConfiguration();
  }

  /**
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   */
  protected function save(ContentEntityInterface $entity): void {
    if (empty($entity->eca_context) || ($this->configuration['save_entity'] === PluginConditionBase::OPTION_YES)) {
      $entity->save();
    }
  }

  /**
   * {@inheritdoc}
   */
  public function execute($entity = NULL) {
    if ($entity === NULL) {
      return;
    }
    if ($this->configuration['load_entity_into_token'] === PluginConditionBase::OPTION_YES) {
      $this->tokenServices->addTokenData($entity->getEntityTypeId(), $entity);
    }
    if ($this->configuration['load_current_user_into_token'] === PluginConditionBase::OPTION_YES) {
      $this->tokenServices->addTokenData('user', $this->currentUser);
    }
    foreach ($this->getFieldsToUpdate() as $field => $value) {
      $output = $this->tokenServices->replace($value);
      if ($this->configuration['strip_tags'] === PluginConditionBase::OPTION_YES) {
        $output = preg_replace('/[\t\n\r\0\x0B]/', '', strip_tags($output));
      }
      if ($this->configuration['trim'] === PluginConditionBase::OPTION_YES) {
        $output = trim($output);
      }
      $entity->$field = $output;
    }
    $this->save($entity);
  }

  /**
   * {@inheritdoc}
   */
  public function access($object, AccountInterface $account = NULL, $return_as_object = FALSE) {
    /** @var \Drupal\Core\Access\AccessResultInterface $result */
    $result = $object->access('update', $account, TRUE);

    foreach ($this->getFieldsToUpdate() as $field => $value) {
      $result->andIf($object->{$field}->access('edit', $account, TRUE));
    }

    return $return_as_object ? $result : $result->isAllowed();
  }

}
