<?php

namespace Drupal\eca_content\Plugin\ECA\Event;

use Drupal;
use Drupal\eca\Annotation\EcaEvent;
use Drupal\eca\PluginEventBase;
use Drupal\eca_content\Event\ContentEntityBase;
use Drupal\eca_content\Event\ContentEntityBundleCreate;
use Drupal\eca_content\Event\ContentEntityBundleDelete;
use Drupal\eca_content\Event\ContentEntityCreate;
use Drupal\eca_content\Event\ContentEntityDelete;
use Drupal\eca_content\Event\ContentEntityEvents;
use Drupal\eca_content\Event\ContentEntityFieldValuesInit;
use Drupal\eca_content\Event\ContentEntityInsert;
use Drupal\eca_content\Event\ContentEntityLoad;
use Drupal\eca_content\Event\ContentEntityPreDelete;
use Drupal\eca_content\Event\ContentEntityPreLoad;
use Drupal\eca_content\Event\ContentEntityPrepareForm;
use Drupal\eca_content\Event\ContentEntityPrepareView;
use Drupal\eca_content\Event\ContentEntityPreSave;
use Drupal\eca_content\Event\ContentEntityRevisionCreate;
use Drupal\eca_content\Event\ContentEntityRevisionDelete;
use Drupal\eca_content\Event\ContentEntityStorageLoad;
use Drupal\eca_content\Event\ContentEntityTranslationCreate;
use Drupal\eca_content\Event\ContentEntityTranslationDelete;
use Drupal\eca_content\Event\ContentEntityTranslationInsert;
use Drupal\eca_content\Event\ContentEntityUpdate;
use Drupal\eca_content\Event\ContentEntityView;

/**
 * Plugin implementation of the ECA Events for content entities.
 *
 * @EcaEvent(
 *   id = "content_entity",
 *   deriver = "Drupal\eca_content\Plugin\ECA\Event\ContentEntityEventDeriver"
 * )
 */
class ContentEntityEvent extends PluginEventBase {

  /**
   * {@inheritdoc}
   */
  public function applies($event, array $arguments): bool {
    if ($event instanceof ContentEntityBase) {
      $entity = $event->getEntity();
      if (empty($arguments['type'])) {
        return TRUE;
      }
      if ($entity->getEntityTypeId() === $arguments['type']) {
        if (empty($arguments['bundle'])) {
          return TRUE;
        }
        return ($entity->bundle() === $arguments['bundle']);
      }
    }
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function output($event): array {
    return [
      'entity' => $event->getEntity(),
    ];
  }

  /**
   * @return array[]
   */
  public static function actions(): array {
    return [
      'bundlecreate' => [
        'label' => 'Create bundle',
        'drupal_id' => ContentEntityEvents::BUNDLECREATE,
        'drupal_event_class' => ContentEntityBundleCreate::class,
      ],
      'bundledelete' => [
        'label' => 'Delete bundle',
        'drupal_id' => ContentEntityEvents::BUNDLEDELETE,
        'drupal_event_class' => ContentEntityBundleDelete::class,
      ],
      'create' => [
        'label' => 'Create',
        'drupal_id' => ContentEntityEvents::CREATE,
        'drupal_event_class' => ContentEntityCreate::class,
      ],
      'revisioncreate' => [
        'label' => 'Create revision',
        'drupal_id' => ContentEntityEvents::REVISIONCREATE,
        'drupal_event_class' => ContentEntityRevisionCreate::class,
      ],
      'preload' => [
        'label' => 'Preload',
        'drupal_id' => ContentEntityEvents::PRELOAD,
        'drupal_event_class' => ContentEntityPreLoad::class,
      ],
      'load' => [
        'label' => 'Load',
        'drupal_id' => ContentEntityEvents::LOAD,
        'drupal_event_class' => ContentEntityLoad::class,
      ],
      'storageload' => [
        'label' => 'Load from storage',
        'drupal_id' => ContentEntityEvents::STORAGELOAD,
        'drupal_event_class' => ContentEntityStorageLoad::class,
      ],
      'presave' => [
        'label' => 'Presave',
        'drupal_id' => ContentEntityEvents::PRESAVE,
        'drupal_event_class' => ContentEntityPreSave::class,
      ],
      'insert' => [
        'label' => 'Insert',
        'drupal_id' => ContentEntityEvents::INSERT,
        'drupal_event_class' => ContentEntityInsert::class,
      ],
      'update' => [
        'label' => 'Update',
        'drupal_id' => ContentEntityEvents::UPDATE,
        'drupal_event_class' => ContentEntityUpdate::class,
      ],
      'translationcreate' => [
        'label' => 'Create translation',
        'drupal_id' => ContentEntityEvents::TRANSLATIONCREATE,
        'drupal_event_class' => ContentEntityTranslationCreate::class,
      ],
      'translationinsert' => [
        'label' => 'Insert translation',
        'drupal_id' => ContentEntityEvents::TRANSLATIONINSERT,
        'drupal_event_class' => ContentEntityTranslationInsert::class,
      ],
      'translationdelete' => [
        'label' => 'Delete translation',
        'drupal_id' => ContentEntityEvents::TRANSLATIONDELETE,
        'drupal_event_class' => ContentEntityTranslationDelete::class,
      ],
      'predelete' => [
        'label' => 'Predelete',
        'drupal_id' => ContentEntityEvents::PREDELETE,
        'drupal_event_class' => ContentEntityPreDelete::class,
      ],
      'delete' => [
        'label' => 'Delete',
        'drupal_id' => ContentEntityEvents::DELETE,
        'drupal_event_class' => ContentEntityDelete::class,
      ],
      'revisiondelete' => [
        'label' => 'Delete revision',
        'drupal_id' => ContentEntityEvents::REVISIONDELETE,
        'drupal_event_class' => ContentEntityRevisionDelete::class,
      ],
      'view' => [
        'label' => 'View',
        'drupal_id' => ContentEntityEvents::VIEW,
        'drupal_event_class' => ContentEntityView::class,
      ],
      'prepareview' => [
        'label' => 'Prepare view',
        'drupal_id' => ContentEntityEvents::PREPAREVIEW,
        'drupal_event_class' => ContentEntityPrepareView::class,
      ],
      'prepareform' => [
        'label' => 'Prepare form',
        'drupal_id' => ContentEntityEvents::PREPAREFORM,
        'drupal_event_class' => ContentEntityPrepareForm::class,
      ],
      'fieldvaluesinit' => [
        'label' => 'Init field values',
        'drupal_id' => ContentEntityEvents::FIELDVALUESINIT,
        'drupal_event_class' => ContentEntityFieldValuesInit::class,
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function fields(): array {
    $fields = [];
    if (!empty($this->pluginDefinition['entity_type_id'])) {
      $fields[] = [
        'name' => 'type',
        'label' => 'Type',
        'type' => 'String',
        'value' => $this->pluginDefinition['entity_type_id'],
        'editable' => FALSE,
      ];
      $bundleChoices = [[
        'name' => '- any -',
        'value' => '',
      ]];
      /** @var \Drupal\Core\Entity\EntityTypeBundleInfoInterface $bundleInfo */
      $bundleInfo = Drupal::service('entity_type.bundle.info');
      foreach ($bundleInfo->getBundleInfo($this->pluginDefinition['entity_type_id']) as $bundle => $bundleDef) {
        $bundleChoices[] = [
          'name' => $bundleDef['label'],
          'value' => $bundle,
        ];
      }
      $fields[] = [
        'name' => 'bundle',
        'label' => 'Bundle',
        'type' => 'Dropdown',
        'extras' => [
          'choices' => $bundleChoices,
        ],
      ];
    }
    return $fields;
  }

}
