<?php

namespace Drupal\eca\Entity;

use Drupal;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\eca\Entity\Objects\EcaEvent;
use Drupal\eca\Entity\Objects\EcaObject;
use Drupal\eca\Entity\Objects\EcaAction;
use Drupal\Core\Config\Entity\ConfigEntityBase;
use Drupal\eca\PluginModelerInterface;

/**
 * Defines the ECA entity type.
 *
 * @ConfigEntityType(
 *   id = "eca",
 *   label = @Translation("ECA"),
 *   label_collection = @Translation("ECAs"),
 *   label_singular = @Translation("ECA"),
 *   label_plural = @Translation("ECAs"),
 *   label_count = @PluralTranslation(
 *     singular = "@count ECA",
 *     plural = "@count ECAs",
 *   ),
 *   config_prefix = "eca",
 *   entity_keys = {
 *     "id" = "id",
 *     "modeler" = "modeler",
 *     "label" = "label",
 *     "uuid" = "uuid",
 *     "filename" = "filename",
 *     "status" = "status"
 *   },
 *   config_export = {
 *     "id",
 *     "modeler",
 *     "label",
 *     "uuid",
 *     "filename",
 *     "status",
 *     "version",
 *     "events",
 *     "conditions",
 *     "actions"
 *   }
 * )
 */
class Eca extends ConfigEntityBase {

  /**
   * The ID.
   *
   * @var string
   */
  protected string $id;

  /**
   * The helpdesk label.
   *
   * @var string
   */
  protected string $label;

  /**
   * @var array
   */
  protected array $events;

  /**
   * @var array
   */
  protected array $conditions;

  /**
   * @var array
   */
  protected array $actions;

  /**
   * @param $id
   *
   * @return string
   */
  protected function buildCacheId($id): string {
    return "eca:{$this->id}:$id";
  }

  /**
   * @return bool
   */
  public function isEditable(): bool {
    return $this->getModeler()->isEditable();
  }

  /**
   * @return \Drupal\eca\PluginModelerInterface
   */
  public function getModeler(): PluginModelerInterface {
    return \Drupal::service('plugin.manager.eca.modeler')->createInstance($this->get('modeler'));
  }

  /**
   * @param string $id
   * @param string $plugin_id
   * @param array $fields
   *
   * @return $this
   */
  public function addCondition(string $id, string $plugin_id, array $fields): Eca {
    $this->conditions[$id] = [
      'plugin' => $plugin_id,
      'fields' => $fields,
    ];
    return $this;
  }

  /**
   * @param string $id
   * @param string $plugin_id
   * @param string $label
   * @param array $fields
   * @param array $successors
   *
   * @return $this
   */
  public function addEvent(string $id, string $plugin_id, string $label, array $fields, array $successors): Eca {
    if (empty($label)) {
      $label = $id;
    }
    $this->events[$id] = [
      'plugin' => $plugin_id,
      'label' => $label,
      'fields' => $fields,
      'successors' => $successors,
    ];
    return $this;
  }

  /**
   * @param string $id
   * @param string $plugin_id
   * @param string $label
   * @param array $fields
   * @param array $successors
   *
   * @return $this
   */
  public function addAction(string $id, string $plugin_id, string $label, array $fields, array $successors): Eca {
    if (empty($label)) {
      $label = $id;
    }
    $this->actions[$id] = [
      'plugin' => $plugin_id,
      'label' => $label,
      'fields' => $fields,
      'successors' => $successors,
    ];
    return $this;
  }

  /**
   * @return \Drupal\eca\Entity\Objects\EcaEvent[]
   */
  public function getUsedEvents(): array {
    $memoryCache = Drupal::service('eca.memory_cache');
    if ($cached = $memoryCache->get($this->buildCacheId('events'))) {
      return $cached->data;
    }

    $events = [];
    foreach ($this->events as $id => $def) {
      if ($event = $this->getEcaObject('event', $def['plugin'], $id, $def['fields'], $def['successors'])) {
        $events[$id] = $event;
      }
    }
    $memoryCache->set($this->buildCacheId('events'), $events, CacheBackendInterface::CACHE_PERMANENT, ['eca.memory_cache:' . $this->id]);
    return $events;
  }

  /**
   * @param \Drupal\eca\Entity\Objects\EcaObject $eca_object
   * @param $condition_id
   *
   * @return bool
   */
  private function assertCondition(EcaObject $eca_object, $condition_id): bool {
    if ($condition_id === FALSE) {
      return TRUE;
    }
    if ($condition = $this->conditions[$condition_id]) {
      /** @var \Drupal\Core\Logger\LoggerChannelInterface $logger */
      $logger = Drupal::service('logger.channel.eca');
      /** @var \Drupal\eca\PluginConditionInterface $plugin */
      if ($plugin = Drupal::service('plugin.manager.eca.condition')->createInstance($condition['plugin'], $condition['fields'])) {
        $plugin->setArguments($eca_object->getVariables());
        if ($plugin->assertCondition()) {
          $logger->info('Asserted condition @id', ['@id' => $condition_id]);
          return TRUE;
        }
        $logger->info('Not asserting condition @id', ['@id' => $condition_id]);
      }
      else {
        $logger->warning('Condition @id not defined', ['@id' => $condition_id]);
      }
      return FALSE;
    }
    return TRUE;
  }

  /**
   * @return \Drupal\eca\Entity\Objects\EcaObject[]
   */
  public function getSuccessors(EcaObject $eca_object): array {
    $successors = [];
    foreach ($eca_object->getSuccessors() as $successor) {
      if ($action = $this->actions[$successor['id']]) {
        if ($successorObject = $this->getEcaObject('action', $action['plugin'], $successor['id'], $action['fields'], $action['successors'], $eca_object->getEvent())) {
          if ($this->assertCondition($eca_object, $successor['condition'])) {
            $successors[] = $successorObject;
          }
        }
      }
    }
    return $successors;
  }

  /**
   * @param string $type
   * @param string $plugin_id
   * @param string $id
   * @param array $fields
   * @param array $successors
   * @param \Drupal\eca\Entity\Objects\EcaEvent|null $event
   *
   * @return \Drupal\eca\Entity\Objects\EcaObject|null
   */
  private function getEcaObject(string $type, string $plugin_id, string $id, array $fields, array $successors, EcaEvent $event = NULL): ?EcaObject {
    $ecaObject = NULL;
    switch ($type) {
      case 'event':
        if ($plugin = Drupal::service('plugin.manager.eca.event')->createInstance($plugin_id, $fields)) {
          $ecaObject = new EcaEvent($this, $id, $plugin);
        }
        break;

      case 'action':
        if ($event !== NULL && $plugin = Drupal::service('plugin.manager.action')->createInstance($plugin_id, $fields)) {
          $ecaObject = new EcaAction($this, $id, $event, $plugin);
        }
        break;

    }
    if ($ecaObject !== NULL) {
      foreach ($fields as $key => $value) {
        $ecaObject->setConfiguration($key, $value);
      }
      $ecaObject->setSuccessors($successors);
      return $ecaObject;
    }
    return NULL;
  }

}
