<?php

namespace Drupal\eca;

use Drupal\Component\Plugin\Exception\PluginException;
use Drupal\Core\Entity\EntityStorageException;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Psr\Log\LoggerInterface;
use function Sodium\compare;

class ModelerServices {

  use PluginTrait;

  /**
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected EntityStorageInterface $storage;

  /**
   * @var \Drupal\eca\PluginManagerModeler
   */
  protected PluginManagerModeler $pluginManagerModeler;

  /**
   * @var \Drupal\eca\PluginManagerEvent
   */
  protected PluginManagerEvent $pluginManagerEvent;

  /**
   * @var \Drupal\eca\PluginManagerCondition
   */
  protected PluginManagerCondition $pluginManagerCondition;

  /**
   * @var \Drupal\eca\ActionServices
   */
  protected ActionServices $actionServices;

  /**
   * @var \Psr\Log\LoggerInterface
   */
  protected LoggerInterface $logger;

  /**
   * ECACommands constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   * @param \Drupal\eca\PluginManagerModeler $plugin_manager_modeler
   * @param \Drupal\eca\PluginManagerEvent $plugin_manager_event
   * @param \Drupal\eca\PluginManagerCondition $plugin_manager_condition
   * @param \Drupal\eca\ActionServices $action_services
   * @param \Psr\Log\LoggerInterface $logger
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, PluginManagerModeler $plugin_manager_modeler, PluginManagerEvent $plugin_manager_event, PluginManagerCondition $plugin_manager_condition, ActionServices $action_services, LoggerInterface $logger) {
    $this->storage = $entity_type_manager->getStorage('eca');
    $this->pluginManagerModeler = $plugin_manager_modeler;
    $this->pluginManagerEvent = $plugin_manager_event;
    $this->pluginManagerCondition = $plugin_manager_condition;
    $this->actionServices = $action_services;
    $this->logger = $logger;
  }

  /**
   * Import a single ECA file.
   *
   * @param \Drupal\eca\PluginModelerInterface $modeler
   * @param string $filename
   */
  public function import(PluginModelerInterface $modeler, string $filename): void {
    $id = mb_strtolower($modeler->getId());
    /** @var \Drupal\eca\Entity\Eca $config */
    $config = $this->storage->load($id);
    if ($config === NULL) {
      $config = $this->storage->create([
        'id' => $id,
        'modeler' => $modeler->getPluginId(),
      ]);
    }
    $config->set('filename', $filename);
    $config->set('label', $modeler->getLabel());
    $config->set('status', $modeler->getStatus());
    $config->set('version', $modeler->getVersion());
    $config->set('events', []);
    $config->set('conditions', []);
    $config->set('actions', []);
    $modeler->readComponents($config);
    try {
      $config->save();
    }
    catch (EntityStorageException $e) {
      // TODO: Log this exception.
    }
  }

  /**
   * Update all previously imported models.
   */
  public function updateAll(): void {
    /** @var \Drupal\eca\Entity\Eca $eca */
    foreach ($this->storage->loadMultiple() as $eca) {
      $modeler = $this->getModeler($eca->get('modeler'));
      if ($modeler === NULL) {
        $this->logger->error('This modeler plugin ' . $eca->get('modeler') . ' does not exist.');
        continue;
      }
      if ($modeler->isEditable()) {
        // Editable models have no external files.
        continue;
      }
      $filename = $eca->get('filename');
      if (!file_exists($filename)) {
        $this->logger->error('This file '. $eca->get('filename') . ' does not exist.');
        continue;
      }
      $modeler->load(file_get_contents($filename));
      $this->import($modeler, $filename);
    }
  }

  /**
   * @param $plugin_id
   *
   * @return \Drupal\eca\PluginModelerInterface|null
   */
  public function getModeler($plugin_id): ?PluginModelerInterface {
    /** @var \Drupal\eca\PluginModelerInterface $modeler */
    try {
      $modeler = $this->pluginManagerModeler->createInstance($plugin_id);
    }
    catch (PluginException $e) {
      return NULL;
    }
    return $modeler;
  }

  /**
   * @return \Drupal\eca\PluginEventInterface[]
   */
  public function events(): array {
    static $events;
    if ($events === NULL) {
      $events = [];
      foreach ($this->pluginManagerEvent->getDefinitions() as $plugin_id => $definition) {
        try {
          $events[] = $this->pluginManagerEvent->createInstance($plugin_id);
        }
        catch (PluginException $e) {
          // Can be ignored
        }
      }
    }
    $this->sortPlugins($events);
    return $events;
  }

  /**
   * @return \Drupal\eca\PluginConditionInterface[]
   */
  public function conditions(): array {
    static $conditions;
    if ($conditions === NULL) {
      $conditions = [];
      foreach ($this->pluginManagerCondition->getDefinitions() as $plugin_id => $definition) {
        try {
          $conditions[] = $this->pluginManagerCondition->createInstance($plugin_id);
        }
        catch (PluginException $e) {
          // Can be ignored
        }
      }
    }
    $this->sortPlugins($conditions);
    return $conditions;
  }

  /**
   * Export components for all ECA modelers.
   */
  public function export(): void {
    foreach ($this->pluginManagerModeler->getDefinitions() as $plugin_id => $definition) {
      try {
        /** @var \Drupal\eca\PluginModelerInterface $modeler */
        $modeler = $this->pluginManagerModeler->createInstance($plugin_id);
        $modeler->export();
      }
      catch (PluginException $e) {
        // Can be ignored
      }
    }
  }

}
