<?php

namespace Drupal\eca\Plugin\ECA\Condition;

use Drupal\eca\PluginConditionBase;

/**
 * Base class for ECA condition plugins to compare strings.
 */
abstract class BaseStringComparison extends PluginConditionBase {

  public const COMPARE_EQUALS = 'equal';
  public const COMPARE_BEGINS_WITH = 'beginswith';
  public const COMPARE_ENDS_WITH = 'endswith';
  public const COMPARE_CONTAINS = 'contains';

  /**
   * Get the first string value for comparison.
   *
   * @return string
   */
  abstract protected function getFirstValue(): string;

  /**
   * Get the second string value for comparison.
   *
   * @return string
   */
  abstract protected function getSecondValue(): string;

  /**
   * @return bool
   */
  protected function caseSensitive(): bool {
    return $this->configuration['case'] === self::OPTION_YES;
  }

  /**
   * {@inheritdoc}
   */
  final public function assertConditions(): bool {
    $value1 = $this->tokenServices->replace($this->getFirstValue());
    $value2 = $this->tokenServices->replace($this->getSecondValue());
    if (!$this->caseSensitive()) {
      $value1 = strtolower($value1);
      $value2 = strtolower($value2);
    }
    switch ($this->configuration['type']) {
      case self::COMPARE_EQUALS:
        return $value1 === $value2;

      case self::COMPARE_BEGINS_WITH:
        return strpos($value1, $value2) === 0;

      case self::COMPARE_ENDS_WITH:
        return strpos($value1, $value2) === (strlen($value1) - strlen($value2));

      case self::COMPARE_CONTAINS:
        return strpos($value1, $value2) !== FALSE;

    }
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function fields(): array {
    $fields = parent::fields();
    $fields[] = [
      'name' => 'case',
      'label' => 'Case sensitive comparison',
      'type' => 'Dropdown',
      'value' => self::OPTION_NO,
      'extras' => [
        'choices' => [
          [
            'name' => 'no',
            'value' => self::OPTION_NO,
          ],
          [
            'name' => 'yes',
            'value' => self::OPTION_YES,
          ],
        ],
      ],
    ];
    $fields[] = [
      'name' => 'type',
      'label' => 'Comparison type',
      'type' => 'Dropdown',
      'value' => self::COMPARE_EQUALS,
      'extras' => [
        'choices' => [
          [
            'name' => 'equals',
            'value' => self::COMPARE_EQUALS,
          ],
          [
            'name' => 'begins with',
            'value' => self::COMPARE_BEGINS_WITH,
          ],
          [
            'name' => 'ends with',
            'value' => self::COMPARE_ENDS_WITH,
          ],
          [
            'name' => 'contains',
            'value' => self::COMPARE_CONTAINS,
          ],
        ],
      ],
    ];
    return $fields;
  }

}
