<?php

namespace Drupal\eca\Plugin\ECA\Event;

use Drupal\block_content\BlockContentEvents;
use Drupal\block_content\Event\BlockContentGetDependencyEvent;
use Drupal\eca\Annotation\EcaEvent;
use Drupal\eca\Event\FormBase;
use Drupal\eca\Event\FormBuild;
use Drupal\eca\Event\FormSubmit;
use Drupal\eca\Event\FormValidate;
use Drupal\eca\PluginEventBase;
use Drupal\config_translation\Event\ConfigMapperPopulateEvent;
use Drupal\config_translation\Event\ConfigTranslationEvents;
use Drupal\Core\File\Event\FileUploadSanitizeNameEvent;
use Drupal\Core\Render\PageDisplayVariantSelectionEvent;
use Drupal\Core\Render\RenderEvents;
use Drupal\Core\Session\AccountEvents;
use Drupal\Core\Session\AccountSetEvent;
use Drupal\jsonapi\ResourceType\ResourceTypeBuildEvent;
use Drupal\jsonapi\ResourceType\ResourceTypeBuildEvents;
use Drupal\language\Config\LanguageConfigOverrideCrudEvent;
use Drupal\language\Config\LanguageConfigOverrideEvents;
use Drupal\layout_builder\Event\PrepareLayoutEvent;
use Drupal\layout_builder\Event\SectionComponentBuildRenderArrayEvent;
use Drupal\layout_builder\LayoutBuilderEvents;
use Drupal\locale\LocaleEvent;
use Drupal\locale\LocaleEvents;
use Drupal\migrate\Event\MigrateEvents;
use Drupal\migrate\Event\MigrateIdMapMessageEvent;
use Drupal\migrate\Event\MigrateImportEvent;
use Drupal\migrate\Event\MigrateMapDeleteEvent;
use Drupal\migrate\Event\MigrateMapSaveEvent;
use Drupal\migrate\Event\MigratePostRowSaveEvent;
use Drupal\migrate\Event\MigratePreRowSaveEvent;
use Drupal\migrate\Event\MigrateRollbackEvent;
use Drupal\migrate\Event\MigrateRowDeleteEvent;

/**
 * Plugin implementation of the ECA Events for Drupal core.
 *
 * @EcaEvent(
 *   id = "drupal",
 *   deriver = "Drupal\eca\Plugin\ECA\Event\DrupalCoreEventDeriver"
 * )
 */
class DrupalCoreEvent extends PluginEventBase {

  public const FORMBUILD = 'eca.content_entity.formbuild';

  public const FORMVALIDATE = 'eca.content_entity.formvalidate';

  public const FORMSUBMIT = 'eca.content_entity.formsubmit';

  /**
   * @inheritDoc
   */
  public function applies($event, array $arguments): bool {
    if ($event instanceof FormBase) {
      return $event->getForm()['#form_id'] === $arguments['id'];
    }
    return parent::applies($event, $arguments);
  }

  /**
   * {@inheritdoc}
   */
  public function output($event): array {
    if ($event instanceof FormBase) {
      return [
        'form' => $event->getForm(),
        'form_state' => $event->getFormState(),
      ];
    }
    return [];
  }

  /**
   * @return array[]
   */
  public static function actions(): array {
    $actions = [];
    $actions['form_build'] = [
      'label' => 'Build form',
      'drupal_id' => self::FORMBUILD,
      'drupal_event_class' => FormBuild::class,
    ];
    $actions['form_validate'] = [
      'label' => 'Validate form',
      'drupal_id' => self::FORMVALIDATE,
      'drupal_event_class' => FormValidate::class,
    ];
    $actions['form_submit'] = [
      'label' => 'Submit form',
      'drupal_id' => self::FORMSUBMIT,
      'drupal_event_class' => FormSubmit::class,
    ];
    if (class_exists(BlockContentEvents::class)) {
      $actions['block_content_get_dependency'] = [
        'label' => 'Block content get dependency',
        'drupal_id' => BlockContentEvents::BLOCK_CONTENT_GET_DEPENDENCY,
        'drupal_event_class' => BlockContentGetDependencyEvent::class,
      ];
    }
    if (class_exists(ConfigTranslationEvents::class)) {
      $actions['populate_mapper'] = [
        'label' => 'Config manager populated',
        'drupal_id' => ConfigTranslationEvents::POPULATE_MAPPER,
        'drupal_event_class' => ConfigMapperPopulateEvent::class,
      ];
    }
    if (class_exists(FileUploadSanitizeNameEvent::class)) {
      $actions['file_upload_sanitize_name_event'] = [
        'label' => 'Sanitize file name',
        'drupal_id' => FileUploadSanitizeNameEvent::class,
        'drupal_event_class' => FileUploadSanitizeNameEvent::class,
      ];
    }
    if (class_exists(RenderEvents::class)) {
      $actions['select_page_display_variant'] = [
        'label' => 'Select page display mode',
        'drupal_id' => RenderEvents::SELECT_PAGE_DISPLAY_VARIANT,
        'drupal_event_class' => PageDisplayVariantSelectionEvent::class,
      ];
    }
    if (class_exists(AccountEvents::class)) {
      $actions['set_user'] = [
        'label' => 'Set current user',
        'drupal_id' => AccountEvents::SET_USER,
        'drupal_event_class' => AccountSetEvent::class,
      ];
    }
    if (class_exists(ResourceTypeBuildEvents::class)) {
      $actions['build'] = [
        'label' => 'Build resource type',
        'drupal_id' => ResourceTypeBuildEvents::BUILD,
        'drupal_event_class' => ResourceTypeBuildEvent::class,
      ];
    }
    if (class_exists(LanguageConfigOverrideEvents::class)) {
      $actions['delete_override'] = [
        'label' => 'Delete config override',
        'drupal_id' => LanguageConfigOverrideEvents::DELETE_OVERRIDE,
        'drupal_event_class' => LanguageConfigOverrideCrudEvent::class,
      ];
      $actions['save_override'] = [
        'label' => 'Save config override',
        'drupal_id' => LanguageConfigOverrideEvents::SAVE_OVERRIDE,
        'drupal_event_class' => LanguageConfigOverrideCrudEvent::class,
      ];
    }
    if (class_exists(LayoutBuilderEvents::class)) {
      $actions['prepare_layout'] = [
        'label' => 'Prepare layout builder element',
        'drupal_id' => LayoutBuilderEvents::PREPARE_LAYOUT,
        'drupal_event_class' => PrepareLayoutEvent::class,
      ];
      $actions['section_component_build_render_array'] = [
        'label' => 'Build render array',
        'drupal_id' => LayoutBuilderEvents::SECTION_COMPONENT_BUILD_RENDER_ARRAY,
        'drupal_event_class' => SectionComponentBuildRenderArrayEvent::class,
      ];
    }
    if (class_exists(LocaleEvents::class)) {
      $actions['save_translation'] = [
        'label' => 'Save translated string',
        'drupal_id' => LocaleEvents::SAVE_TRANSLATION,
        'drupal_event_class' => LocaleEvent::class,
      ];
    }
    if (class_exists(MigrateEvents::class)) {
      $actions['idmap_message'] = [
        'label' => 'Save message to ID map',
        'drupal_id' => MigrateEvents::IDMAP_MESSAGE,
        'drupal_event_class' => MigrateIdMapMessageEvent::class,
      ];
      $actions['map_delete'] = [
        'label' => 'Remove entry from migration map',
        'drupal_id' => MigrateEvents::MAP_DELETE,
        'drupal_event_class' => MigrateMapDeleteEvent::class,
      ];
      $actions['map_save'] = [
        'label' => 'Save to migration map',
        'drupal_id' => MigrateEvents::MAP_SAVE,
        'drupal_event_class' => MigrateMapSaveEvent::class,
      ];
      $actions['post_import'] = [
        'label' => 'Migration import finished',
        'drupal_id' => MigrateEvents::POST_IMPORT,
        'drupal_event_class' => MigrateImportEvent::class,
      ];
      $actions['post_rollback'] = [
        'label' => 'Migration rollback finished',
        'drupal_id' => MigrateEvents::POST_ROLLBACK,
        'drupal_event_class' => MigrateRollbackEvent::class,
      ];
      $actions['post_row_delete'] = [
        'label' => 'Migration row deleted',
        'drupal_id' => MigrateEvents::POST_ROW_DELETE,
        'drupal_event_class' => MigrateRowDeleteEvent::class,
      ];
      $actions['post_row_save'] = [
        'label' => 'Migration row saved',
        'drupal_id' => MigrateEvents::POST_ROW_SAVE,
        'drupal_event_class' => MigratePostRowSaveEvent::class,
      ];
      $actions['pre_import'] = [
        'label' => 'Migration import started',
        'drupal_id' => MigrateEvents::PRE_IMPORT,
        'drupal_event_class' => MigrateImportEvent::class,
      ];
      $actions['pre_rollback'] = [
        'label' => 'Migration rollback started',
        'drupal_id' => MigrateEvents::PRE_ROLLBACK,
        'drupal_event_class' => MigrateRollbackEvent::class,
      ];
      $actions['pre_row_delete'] = [
        'label' => 'Deleting migration row',
        'drupal_id' => MigrateEvents::PRE_ROW_DELETE,
        'drupal_event_class' => MigrateRowDeleteEvent::class,
      ];
      $actions['pre_row_save'] = [
        'label' => 'Saving migration row',
        'drupal_id' => MigrateEvents::PRE_ROW_SAVE,
        'drupal_event_class' => MigratePreRowSaveEvent::class,
      ];
    }
    return $actions;
  }

  /**
   * {@inheritdoc}
   */
  public function fields(): array {
    $fields = [];
    if (in_array($this->pluginDefinition['drupal_id'], [self::FORMBUILD, self::FORMVALIDATE, self::FORMSUBMIT], TRUE)) {
      $fields[] = [
        'name' => 'id',
        'label' => 'Form ID',
        'type' => 'String',
      ];
    }
    return $fields;
  }

}
