<?php

namespace Drupal\eca;

use Drupal\Component\Plugin\PluginInspectionInterface;
use Drupal\Core\Action\ActionInterface;
use Drupal\eca\Entity\Eca;
use Mtownsend\XmlToArray\XmlToArray;

/**
 *
 */
abstract class PluginModelerBpmnBase extends PluginModelerBase {

  /**
   * @var array
   */
  protected array $xmlModel;

  /**
   * @return string
   */
  protected function xmlNsPrefix(): string {
    return '';
  }

  /**
   * {@inheritdoc}
   */
  public function load(string $model): PluginModelerInterface {
    $this->xmlModel = XmlToArray::convert($model);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getId(): string {
    return $this->xmlModel[$this->xmlNsPrefix() . 'process']['@attributes']['id'];
  }

  /**
   * {@inheritdoc}
   */
  public function getLabel(): string {
    return $this->xmlModel[$this->xmlNsPrefix() . 'process']['@attributes']['name'] ?? 'noname';
  }

  /**
   * {@inheritdoc}
   */
  public function getStatus(): bool {
    return (bool) ($this->xmlModel[$this->xmlNsPrefix() . 'process']['@attributes']['isExecutable'] ?? 'true');
  }

  /**
   * {@inheritdoc}
   */
  public function getVersion(): string {
    return $this->xmlModel[$this->xmlNsPrefix() . 'process']['@attributes']['versionTag'] ?? '';
  }

  private function getStartEvents(): array {
    $events = $this->xmlModel[$this->xmlNsPrefix() . 'process'][$this->xmlNsPrefix() . 'startEvent'] ?? [];
    if (isset($events['@attributes'])) {
      return [$events];
    }
    return $events;
  }

  private function getTasks(): array {
    $events = $this->xmlModel[$this->xmlNsPrefix() . 'process'][$this->xmlNsPrefix() . 'task'] ?? [];
    if (isset($events['@attributes'])) {
      return [$events];
    }
    return $events;
  }

  private function getSequenceFlows(): array {
    $events = $this->xmlModel[$this->xmlNsPrefix() . 'process'][$this->xmlNsPrefix() . 'sequenceFlow'] ?? [];
    if (isset($events['@attributes'])) {
      return [$events];
    }
    return $events;
  }

  /**
   * {@inheritdoc}
   */
  public function readComponents(Eca $eca): PluginInspectionInterface {
    $idxExtension = $this->xmlNsPrefix() . 'extensionElements';

    $flow = [];
    foreach ($this->getSequenceFlows() as $sequenceFlow) {
      if (isset($sequenceFlow[$idxExtension])) {
        $condition = $this->findAttribute($sequenceFlow, 'id');
        $eca->addCondition(
          $condition,
          $this->findProperty($sequenceFlow[$idxExtension], 'pluginid'),
          $this->findFields($sequenceFlow[$idxExtension])
        );
      }
      else {
        $condition = FALSE;
      }
      $flow[$this->findAttribute($sequenceFlow, 'sourceRef')][] = [
        'id' => $this->findAttribute($sequenceFlow, 'targetRef'),
        'condition' => $condition,
      ];
    }

    foreach ($this->getStartEvents() as $startEvent) {
      $extension = $startEvent[$idxExtension] ?? [];
      $eca->addEvent(
        $this->findAttribute($startEvent, 'id'),
        $this->findProperty($extension, 'pluginid'),
        $this->findAttribute($startEvent, 'name'),
        $this->findFields($extension),
        $flow[$this->findAttribute($startEvent, 'id')] ?? []
      );
    }

    foreach ($this->getTasks() as $task) {
      $extension = $task[$idxExtension] ?? [];
      $eca->addAction(
        $this->findAttribute($task, 'id'),
        $this->findProperty($extension, 'pluginid'),
        $this->findAttribute($task, 'name'),
        $this->findFields($extension),
        $flow[$this->findAttribute($task, 'id')] ?? []
      );
    }

    return $this;
  }

  /**
   * @return array
   */
  protected function getTemplates(): array {
    $templates = [];
    foreach ($this->modelerServices->events() as $event) {
      $templates[] = $this->prepareEventTemplate($event);
    }
    foreach ($this->modelerServices->conditions() as $condition) {
      $templates[] = $this->prepareConditionTemplate($condition);
    }
    foreach ($this->actionServices->actions() as $action) {
      $templates[] = $this->prepareActionTemplate($action);
    }
    return $templates;
  }

  /**
   * {@inheritdoc}
   */
  public function export(): PluginModelerInterface {
    // Nothing to do by default.
    return $this;
  }

  /**
   * @param \Drupal\Component\Plugin\PluginInspectionInterface $plugin
   * @param string $applies_to
   * @param array $fields
   *
   * @return array
   */
  protected function properties(PluginInspectionInterface $plugin, string $applies_to, array $fields): array {
    $properties = [[
      'label' => 'Plugin ID',
      'type' => 'Hidden',
      'value' => $plugin->getPluginId(),
      'binding' => [
        'type' => 'camunda:property',
        'name' => 'pluginid',
      ],
    ]];
    foreach ($fields as $field) {
      $property = [
        'label' => $field['label'],
        'type' => $field['type'],
        'value' => $field['value'] ?? '',
        'editable' => $field['editable'] ?? TRUE,
        'binding' => [
          'type' => 'camunda:field',
          'name' => $field['name'],
        ],
      ];
      if (isset($field['extras'])) {
        /** @noinspection SlowArrayOperationsInLoopInspection */
        $property = array_merge_recursive($property, $field['extras']);
      }
      $properties[] = $property;
    }
    return [
      'name' => (string) $plugin->getPluginDefinition()['label'],
      'id' => 'org.drupal.' . $plugin->getPluginId(),
      'appliesTo' => [$applies_to],
      'properties' => $properties,
    ];
  }

  /**
   * @param \Drupal\eca\PluginEventInterface $event
   *
   * @return array
   */
  protected function prepareEventTemplate(PluginEventInterface $event): array {
    return $this->properties($event, 'bpmn:Event', $event->fields());
  }

  /**
   * @param \Drupal\eca\PluginConditionInterface $condition
   *
   * @return array
   */
  protected function prepareConditionTemplate(PluginConditionInterface $condition): array {
    return $this->properties($condition, 'bpmn:SequenceFlow', $condition->fields());
  }

  /**
   * @param \Drupal\Core\Action\ActionInterface $action
   *
   * @return array
   */
  protected function prepareActionTemplate(ActionInterface $action): array {
    return $this->properties($action, 'bpmn:Task', $this->actionServices->fields($action));
  }

  /**
   * @param array $element
   * @param string $property_name
   *
   * @return string
   */
  protected function findProperty(array $element, string $property_name): string {
    if (isset($element['camunda:properties']['camunda:property'])) {
      $elements = isset($element['camunda:properties']['camunda:property']['@attributes']) ?
        [$element['camunda:properties']['camunda:property']] :
        $element['camunda:properties']['camunda:property'];
      foreach ($elements as $child) {
        if ($child['@attributes']['name'] === $property_name) {
          return $child['@attributes']['value'];
        }
      }
    }
    return '';
  }

  /**
   * @param array $element
   * @param string $attribute_name
   *
   * @return string
   */
  protected function findAttribute(array $element, string $attribute_name): string {
    return $element['@attributes'][$attribute_name] ?? '';
  }

  protected function findFields(array $element): array {
    $fields = [];
    if (isset($element['camunda:field'])) {
      $elements = isset($element['camunda:field']['@attributes']) ? [$element['camunda:field']] : $element['camunda:field'];
      foreach ($elements as $child) {
        $fields[$child['@attributes']['name']] = $child['camunda:string'];
      }
    }
    return $fields;
  }

}
